/*
 * This file is part of decoding.
 * Copyright (C) 2012  Guillaume Quintin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <decoding/decoding.h>

/* Sudan parameters:
   Decoding Reed-Solomon codes beyond the error-correction diameter.
   Lemma 8, page 6. */
int sudan_max_errors(int n2,int k2) {
  double n,k,rkn,e,num,denom;
  n = (double)n2;
  k = (double)k2;
  rkn = floor(sqrt((2.0 * (n + 1.0)) / (k - 1.0) + .25) - .5);
  num = 2.0 * n - (k - 1) * rkn * (rkn + 1);
  denom = 2.0 * (rkn + 1.0);
  e = n - (k - 1) * rkn - ceil(num / denom);
  return (int)e;
  /* return (int)(n - k * ceil(sqrt(n)) - 1.0); */
}

int sudan_weighted_degree(int n,int k,int e) {
  (void)k;
  return n - e - 1;
}

int sudan_list_size(int n2,int k2,int e2) {
  double n,k,e;
  n = (double)n2;
  k = (double)k2;
  e = (double)e2;
  return (int)floor((n - e) / (k - 1));
}

/* Guruswami-Sudan parameters
   LIST DECODING OF ERROR-CORRECTING CODES
   page 110
*/
int guru_sudan_max_errors(int n2,int k2) {
  double n,k;
  n = (double)n2;
  k = (double)k2;
  return (int)ceil(n -sqrt(n * (k - 1.0))) - 1.0;
}

int guru_sudan_mult(int n2,int k2,int e2) {
  double n,km1,e,t,m,num,denom;
  n = (double)n2;
  km1 = (double)(k2 - 1);
  e = (double)e2;
  t = n - e;
  num = km1 * n + sqrt((km1 * km1 * n * n) + 4.0 * (t * t - km1 * n));
  denom = 2.0 * (t * t - km1 * n);
  m = 1.0 + floor(num / denom);
  return (int)m;
}

int guru_sudan_weighted_degree(int n,int k,int e) {
  int m;
  m = guru_sudan_mult(n,k,e);
  return m * (n - e) - 1;
}

int guru_sudan_list_size(int n2,int k2,int e2) {
  double n,k,e,m;
  n = (double)n2;
  k = (double)k2;
  e = (double)e2;
  m = (double)guru_sudan_mult(n,k,e);
  return (int)(ceil((m * (n - e) - 1.0) / (k - 1)) - 1.0);
}

