/*
 * This file is part of decoding.
 * Copyright (C) 2012  Guillaume Quintin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/* structs */
struct ll_upol_shift_fast_pc { /* pc is for PreComputations */
  R **pow;
  int npow;
};

struct ll_upol_shift_car2_pc {
  R *pow;
  int npow;
};

struct ll_dbpol_shift_fast_pc {
  struct ll_upol_shift_fast_pc *powX,*powY;
};

struct ll_dbpol_shift_car2_pc {
  struct ll_upol_shift_car2_pc *powX,*powY;
};

struct ll_lili_vec {
  R *a;
  int na;
  struct ll_lili_vec *next,*prev;
};

/* algos functions */
void ll_array_add(R*,R*,int,R*,int);
void ll_array_add_mul_by_cte(R*,int,R*,int,R);
void ll_dbpol_horner(R,R*,int,int,R,R);
void ll_dbpol_horner_upol(R*,int*, R*,int,int,R*,int);
void ll_dbpol_mul_by_affine(R*,R*,int,int,R);
void ll_dbpol_mul_by_upolX(R*,R*,int,int,R*,int);
void ll_dbpol_mul_by_upolY(R*,R*,int,int,R*,int);
void ll_dbpol_mul_by_Y(R*,R*,int,int);
void ll_dbpol_mul_by_Yn(R*,R*,int,int,int);
void ll_dbpol_mul_naive(R*,R*,int,int,R*,int,int);
void ll_dbpol_mul(R*,R*,int,int,R*,int,int);
void ll_dbpol_print_factorized(FILE*,R*,int,int);
R *ll_dbpol_malloc_random(int,int);
void ll_dbpol_shift_fast_rec(R*,R*,int,int,R,R**,R*);
void ll_dbpol_shift_fast2(R*,R*,int,int,R,R, struct ll_dbpol_shift_fast_pc*);
struct ll_dbpol_shift_fast_pc *ll_dbpol_shift_fast_prepare(int,int,R,R);
void ll_dbpol_shift_fast(R*,R*,int,int,R,R);
void ll_dbpol_shift_fast_finish(struct ll_dbpol_shift_fast_pc*);
void ll_dbpol_shift_car2_rec(R*,int,int,R,R*);
void ll_dbpol_shift_car22(R*,R*,int,int,R,R, struct ll_dbpol_shift_car2_pc*);
struct ll_dbpol_shift_car2_pc *ll_dbpol_shift_car2_prepare(int,int,R,R);
void ll_dbpol_shift_car2(R*,R*,int,int,R,R);
void ll_dbpol_shift_car2_finish(struct ll_dbpol_shift_car2_pc*);
void ll_dbpol_shift_naive(R*,R*,int,int,R,R);
void ll_dbpol_shift(R*,R*,int,int,R,R);
struct ll_lili_vec *ll_dbpol_Xroots_rec(R*,int,int,int,int,int);
struct ll_lili_vec *ll_dbpol_Xroots(R*,int,int,int);
int ll_koetter(R*,int*,int*, R*,int,int, int,int,R*,int);
struct ll_lili_vec *ll_lili_vec_node_malloc(int);
struct ll_lili_vec *ll_lili_vec_node_malloc_copy(R*,int);
void ll_lili_vec_node_free(struct ll_lili_vec*);
void ll_lili_vec_node_attach_after(struct ll_lili_vec*,struct ll_lili_vec*);
void ll_lili_vec_node_attach_before(struct ll_lili_vec*,struct ll_lili_vec*);
void ll_lili_vec_node_detach(struct ll_lili_vec*);
void ll_lili_vec_list_free(struct ll_lili_vec*);
void ll_mat_copy(R*,R*,int,int);
R *ll_mat_malloc(int,int);
R *ll_mat_malloc_zero(int,int);
R *ll_mat_malloc_random(int,int);
void ll_mat_free(R*,int,int);
void ll_mat_transpose(R*,R*,int,int);
void *ll_malloc(int);
void *ll_realloc(void*,int);
void *ll_calloc(int);
void ll_free(void*);
void ll_print(FILE*,const char*,...);
void ll_upol_quorem_naive(R*,int*,R*,R*,int,R*,int);
void ll_upol_quo_naive(R*,R*,int,R*,int);
void ll_upol_rem_naive(R*,int*,R*,int,R*,int);
void ll_upol_quorem(R*,int*,R*,R*,int,R*,int);
void ll_upol_quo(R*,R*,int,R*,int);
void ll_upol_rem(R*,int*,R*,int,R*,int);
void ll_upol_gcd_naive(R*,int*,R*,int,R*,int);
void ll_upol_gcd(R*,int*,R*,int,R*,int);
void ll_upol_horner(R,R*,int,R);
void ll_upol_multi_horner(R*,R*,int,R*,int);
void ll_upol_mul_by_affine(R*,R*,int,R);
void ll_upol_mul_by_X(R*,R*,int);
void ll_upol_mul_by_Xn(R*,R*,int,int);
void ll_upol_mul_naive(R*,R*,int,R*,int);
void ll_upol_mul(R*,R*,int,R*,int);
void ll_upol_normalize(R*,int);
void ll_upol_print(FILE*,R*,int);
void ll_upol_random(R*,int);
R *ll_upol_malloc_random(int);
void ll_upol_roots(R*,int*,R*,int);
void ll_upol_shift_fast_rec(R*,R*,int,R,R**,R*);
void ll_upol_shift_fast2(R*,R*,int,R, struct ll_upol_shift_fast_pc*);
struct ll_upol_shift_fast_pc *ll_upol_shift_fast_prepare(int,R);
void ll_upol_shift_fast_finish(struct ll_upol_shift_fast_pc*);
void ll_upol_shift_fast(R*,R*,int,R);
void ll_upol_shift_car2_rec(R*,R*,int,R,R*);
void ll_upol_shift_car22(R*,R*,int,R, struct ll_upol_shift_car2_pc*);
struct ll_upol_shift_car2_pc *ll_upol_shift_car2_prepare(int,R);
void ll_upol_shift_car2_finish(struct ll_upol_shift_car2_pc*);
void ll_upol_shift_car2(R*,R*,int,R);
void ll_upol_shift_naive(R*,R*,int,R);
void ll_upol_shift(R*,R*,int,R);
void ll_upol_square_naive(R*,R*,int);
void ll_upol_square(R*,R*,int);
void ll_die(const char*);
int ll_min(int,int);
int ll_max(int,int);
int ll_ceil_div(int,int);
int ll_log2(int);
int ll_prev_log2(int);
void ll_vec_add(R*,R*,R*,int);
void ll_vec_sub(R*,R*,R*,int);
void ll_vec_sub_sub(R*,R*,R*,int);
void ll_vec_add_mul(R*,R*,R*,int);
void ll_vec_mul_by_cte(R*,R*,int,R);
void ll_vec_add_mul_by_cte(R*,R*,int,R);
void ll_vec_sub_mul(R*,R*,R*,int);
void ll_vec_init(R*,int);
void ll_vec_init_zero(R*,int);
void ll_vec_init_random(R*,int);
void ll_vec_clear(R*,int);
int ll_vec_hamming_weight(R*,int);
void ll_vec_zero(R*,int);
int ll_vec_iszero(R*,int);
void ll_vec_random(R*,int);
void ll_vec_random_hamming_weight(R*,int,int);
void ll_vec_copy(R*,R*,int);
R *ll_vec_malloc(int);
R *ll_vec_malloc_zero(int);
R *ll_vec_malloc_copy(R*,int);
R *ll_vec_malloc_random(int);
void ll_vec_free(R*,int);
void ll_vec_print(FILE*,R*,int);

