/*
 * This file is part of decoding.
 * Copyright (C) 2012  Guillaume Quintin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

void ll_upol_quorem_naive(R *r,int *nr,R *q,R *a,int na,R *b,int nb) {
  R inv;
  int dq,dq2,dr,i,j;
  
  R_init(inv);
  if ( R_inv(inv,b[nb - 1]) ) {
    ll_die("leading coefficient not invertible\n"
           "cannot do Euclidean division");
  }
  dq = na - nb;
  dr = na - 1;
  if ( r != a ) ll_vec_copy(r,a,na);
  for( ; dr >= nb - 1 ; ) {
    R_mul(q[dq],r[dr],inv);
    R_zero(r[dr]);
    dr--;
    for( dq2 = dq - 1,i = dr,j = nb - 2 ; j >= 0 ; i--,j-- ) {
      R_sub_mul(r[i],b[j],q[dq]);
      if ( dq2 >= 0 && R_iszero(r[i]) ) {
        R_zero(q[dq2]);
        dq2--;
        dr--;
      }
      else {
        for( i--,j-- ; j >= 0 ; i--,j-- ) {
          R_sub_mul(r[i],b[j],q[dq]);
        }
        break;
      }
    }
    dq = dq2;
  }
  for( ; dr >=0 && R_iszero(r[dr]) ; dr-- );
  nr[0] = dr + 1;
  R_clear(inv);
}

void ll_upol_quo_naive(R *q,R *a,int na,R *b,int nb) {
  R inv,*r;
  int dq2,dq,dr,i,j;

  r = ll_vec_malloc(na);
  R_init(inv);
  if ( R_inv(inv,b[nb - 1]) ) {
    ll_die("leading coefficient not invertible\n"
           "cannot do Euclidean division");
  }
  dq = na - nb;
  dr = na - 1;
  ll_vec_copy(r,a,na);
  for( ; dr >= nb - 1 ; ) {
    R_mul(q[dq],r[dr],inv);
    R_zero(r[dr]);
    dr--;
    for( dq2 = dq - 1,i = dr,j = nb - 2 ; j >= 0 ; i--,j-- ) {
      R_sub_mul(r[i],b[j],q[dq]);
      if ( dq2 >= 0 && R_iszero(r[i]) ) {
        R_zero(q[dq2]);
        dq2--;
        dr--;
      }
      else {
        for( i--,j-- ; j >= 0 ; i--,j-- ) {
          R_sub_mul(r[i],b[j],q[dq]);
        }
        break;
      }
    }
    dq = dq2;
  }
  R_clear(inv);
  ll_vec_free(r,na);
}

void ll_upol_rem_naive(R *r,int *nr,R *a,int na,R *b,int nb) {
  R inv,q;
  int dr,i,j;
  
  R_init(inv);
  R_init(q);
  if ( R_inv(inv,b[nb - 1]) ) {
    ll_die("leading coefficient not invertible\n"
           "cannot do Euclidean division");
  }
  dr = na - 1;
  if ( r != a ) ll_vec_copy(r,a,na);
  for( ; dr >= nb - 1 ; ) {
    R_mul(q,r[dr],inv);
    R_zero(r[dr]);
    dr--;
    for( i = dr,j = nb - 2 ; j >= 0 ; i--,j-- ) {
      R_sub_mul(r[i],b[j],q);
      if ( R_iszero(r[i]) ) {
        dr--;
      }
      else {
        for( i--,j-- ; j >= 0 ; i--,j--) {
          R_sub_mul(r[i],b[j],q);
        }
        break;
      }
    }
  }
  for( ; dr >=0 && R_iszero(r[dr]) ; dr-- );
  nr[0] = dr + 1;
  R_clear(q);
  R_clear(inv);
}

/* TODO: implement fast euclidean division: Moenck, Borodin */
void ll_upol_quorem(R *r,int *nr,R *q,R *a,int na,R *b,int nb) {
  ll_upol_quorem_naive(r,nr,q,a,na,b,nb);
}

void ll_upol_quo(R *q,R *a,int na,R *b,int nb) {
  ll_upol_quo_naive(q,a,na,b,nb);
}

void ll_upol_rem(R *r,int *nr,R *a,int na,R *b,int nb) {
  ll_upol_rem_naive(r,nr,a,na,b,nb);
}

