/*
 * This file is part of decoding.
 * Copyright (C) 2012  Guillaume Quintin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/* ARITHMETIC */

void ll_vec_add(R *r,R *a,R *b,int n) {
  int i;
  for( i = 0 ; i < n ; i++ ) {
    R_add(r[i],a[i],b[i]);
  }
}

void ll_vec_sub(R *r,R *a,R *b,int n) {
  int i;
  for( i = 0 ; i < n ; i++ ) {
    R_sub(r[i],a[i],b[i]);
  }
}

void ll_vec_sub_sub(R *r,R *a,R *b,int n) {
  int i;
  for( i = 0 ; i < n ; i++ ) {
    R_sub_sub(r[i],a[i],b[i]);
  }
}

void ll_vec_add_mul(R *r,R *a,R *b,int n) {
  int i;
  for( i = 0 ; i < n ; i++ ) {
    R_add_mul(r[i],a[i],b[i]);
  }
}

void ll_vec_mul_by_cte(R *r,R *a,int na,R b) {
  int i;
  for( i = 0 ; i < na ; i++ ) {
    R_mul(r[i],a[i],b);
  }
}

void ll_vec_add_mul_by_cte(R *r,R *a,int na,R b) {
  int i;
  for( i = 0 ; i < na ; i++ ) {
    R_add_mul(r[i],a[i],b);
  }
}

void ll_vec_sub_mul(R *r,R *a,R *b,int n) {
  int i;
  for( i = 0 ; i < n ; i++ ) {
    R_sub_mul(r[i],a[i],b[i]);
  }
}

/* MANIPULATION */

void ll_vec_init(R *a,int na) {
  int i;
  for( i = 0 ; i < na ; i++ ) {
    R_init(a[i]);
  }
}

void ll_vec_init_zero(R *a,int na) {
  int i;
  for( i = 0 ; i < na ; i++ ) {
    R_init(a[i]);
    R_zero(a[i]);
  }
}

void ll_vec_init_random(R *a,int na) {
  int i;
  for( i = 0 ; i < na ; i++ ) {
    R_init(a[i]);
    R_random(a[i]);
  }
}

void ll_vec_clear(R *a,int na) {
  int i;
  for( i = 0 ; i < na ; i++ ) {
    R_clear(a[i]);
  }
}

int ll_vec_hamming_weight(R *a,int na) {
  int i,w;
  w = 0;
  for( i = 0 ; i < na ; i++ ) {
    if ( !R_iszero(a[i]) ) w++;
  }
  return w;
}

void ll_vec_zero(R *a,int na) {
  int i;
  for( i = 0 ; i < na ; i++ ) {
    R_zero(a[i]);
  }
}

int ll_vec_iszero(R *a,int na) {
  int i;
  for( i = 0 ; i < na ; i++ ) {
    if ( !R_iszero(a[i]) ) return 0;
  }
  return -1;
}

void ll_vec_random(R *a,int na) {
  int i;
  for( i = 0 ; i < na ; i++ ) {
    R_random(a[i]);
  }
}

void ll_vec_random_hamming_weight(R *r,int nr,int w) {
  int i,j;
  ll_vec_zero(r,nr);
  j = -1;
  for( i = 0 ; i < w ; i++ ) {
    j = j + 1 + (rand() % (nr  - (j + 1) - (w - i)));
    R_random(r[j]);
    if ( R_iszero(r[j]) ) R_castint(r[j],1);
  }
}

void ll_vec_copy(R *r,R *a,int na) {
  int i;
  for( i = 0 ; i < na ; i++ ) {
    R_copy(r[i],a[i]);
  }
}

/* MALLOC, FREE */

R *ll_vec_malloc(int nr) {
  int i;
  R *r;
  r = ll_malloc(nr * sizeof(R));
  for( i = 0 ; i < nr ; i++ ) {
    R_init(r[i]);
  }
  return r;
}

R *ll_vec_malloc_zero(int nr) {
  int i;
  R *r;
  r = ll_malloc(nr * sizeof(R));
  for( i = 0 ; i < nr ; i++ ) {
    R_init(r[i]);
    R_zero(r[i]);
  }
  return r;
}

R *ll_vec_malloc_copy(R *a,int na) {
  int i;
  R *r;
  r = ll_malloc(na * sizeof(R));
  for( i = 0 ; i < na ; i++ ) {
    R_init(r[i]);
    R_copy(r[i],a[i]);
  }
  return r;
}

R *ll_vec_malloc_random(int nr) {
  int i;
  R *r;
  r = ll_malloc(nr * sizeof(R));
  for( i = 0 ; i < nr ; i++ ) {
    R_init(r[i]);
    R_random(r[i]);
  }
  return r;
}

void ll_vec_free(R *r,int nr) {
  int i;
  for( i = 0 ; i < nr ; i++ ) {
    R_clear(r[i]);
  }
  free(r);
}

int ll_vec_cmp(R *a,R *b,int n) {
  int i;
  for( i = 0 ; i < n ; i++ ) {
    if ( R_cmp(a[i],b[i]) ) return -1;
  }
  return 0;
}

/* PRETTY PRINT */

void ll_vec_print(FILE *out,R *a,int na) {
  int i;
  fputs("[ ",out);
  if ( na > 0 ) {
    R_print(out,a[0]);
    for( i = 1 ; i < na ; i++ ) {
      fputs(", ",out);
      R_print(out,a[i]);
    }
  }
  fputs(" ]",out);
}

