/*
 * This file is part of decoding.
 * Copyright (C) 2012  Guillaume Quintin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef _DECODING_H
#define _DECODING_H

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <math.h>

/* common functions of decoding.c */
extern int sudan_max_errors(int,int);
extern int sudan_weighted_degree(int,int,int);
extern int sudan_list_size(int,int,int);
extern int guru_sudan_max_errors(int,int);
extern int guru_sudan_mult(int,int,int);
extern int guru_sudan_weighted_degree(int,int,int);
extern int guru_sudan_list_size(int,int,int);

/* macro concatenation */
#define __C2(a,b) a##b
#define __C(a,b)  __C2(a,b)

/* ring functions */
#define R_clear __C(R,_clear)
#define R_clears __C(R,_clears)
#define R_init __C(R,_init)
#define R_inits __C(R,_inits)
#define R_copy __C(R,_copy)
#define R_init_copy __C(R,_init_copy)
#define R_shallow_copy __C(R,_shallow_copy)
#define R_print __C(R,_print)
#define R_cmp __C(R,_cmp)
#define R_iszero __C(R,_iszero)
#define R_zero __C(R,_zero)
#define R_random __C(R,_random)
#define R_add __C(R,_add)
#define R_sub __C(R,_sub)
#define R_sub_sub __C(R,_sub_sub)
#define R_add_mul __C(R,_add_mul)
#define R_sub_mul __C(R,_sub_mul)
#define R_neg __C(R,_neg)
#define R_mul __C(R,_mul)
#define R_square __C(R,_square)
#define R_inv __C(R,_inv)
#define R_div __C(R,_div)
#define R_castint __C(R,_castint)

/* structs */
#define ll_upol_shift_fast_pc __C(R,ll_upol_shift_fast_pc)
#define ll_upol_shift_car2_pc __C(R,ll_upol_shift_car2_pc)
#define ll_dbpol_shift_fast_pc __C(R,ll_dbpol_shift_fast_pc)
#define ll_dbpol_shift_car2_pc __C(R,ll_dbpol_shift_car2_pc)
#define ll_lili_vec __C(R,ll_lili_vec)

/* labels */
#define ll_dbpol_Xroots_rec_free_q __C(R,ll_dbpol_Xroots_rec_free_q)
#define ll_dbpol_Xroots_rec_free_z0 __C(R,ll_dbpol_Xroots_rec_free_z0)
#define koetter_free_stuff __C(R,koetter_free_stuff)

/* algos functions */
#define ll_array_add __C(R,_ll_array_add)
#define ll_array_add_mul_by_cte __C(R,_ll_array_add_mul_by_cte)
#define ll_calloc __C(R,_ll_calloc)
#define ll_ceil_div __C(R,_ll_ceil_div)
#define ll_dbpol_Xroots __C(R,_ll_dbpol_Xroots)
#define ll_dbpol_Xroots_rec __C(R,_ll_dbpol_Xroots_rec)
#define ll_dbpol_horner __C(R,_ll_dbpol_horner)
#define ll_dbpol_horner_upol __C(R,_ll_dbpol_horner_upol)
#define ll_dbpol_malloc_random __C(R,_ll_dbpol_malloc_random)
#define ll_dbpol_mul __C(R,_ll_dbpol_mul)
#define ll_dbpol_mul_by_Y __C(R,_ll_dbpol_mul_by_Y)
#define ll_dbpol_mul_by_Yn __C(R,_ll_dbpol_mul_by_Yn)
#define ll_dbpol_mul_by_affine __C(R,_ll_dbpol_mul_by_affine)
#define ll_dbpol_mul_by_upolX __C(R,_ll_dbpol_mul_by_upolX)
#define ll_dbpol_mul_by_upolY __C(R,_ll_dbpol_mul_by_upolY)
#define ll_dbpol_mul_naive __C(R,_ll_dbpol_mul_naive)
#define ll_dbpol_print_factorized __C(R,_ll_dbpol_print_factorized)
#define ll_dbpol_replace_X_by_XYn __C(R,_ll_dbpol_replace_X_by_XYn)
#define ll_dbpol_shift __C(R,_ll_dbpol_shift)
#define ll_dbpol_shift_car2 __C(R,_ll_dbpol_shift_car2)
#define ll_dbpol_shift_car22 __C(R,_ll_dbpol_shift_car22)
#define ll_dbpol_shift_car2_finish __C(R,_ll_dbpol_shift_car2_finish)
#define ll_dbpol_shift_car2_prepare __C(R,_ll_dbpol_shift_car2_prepare)
#define ll_dbpol_shift_car2_rec __C(R,_ll_dbpol_shift_car2_rec)
#define ll_dbpol_shift_fast __C(R,_ll_dbpol_shift_fast)
#define ll_dbpol_shift_fast2 __C(R,_ll_dbpol_shift_fast2)
#define ll_dbpol_shift_fast_finish __C(R,_ll_dbpol_shift_fast_finish)
#define ll_dbpol_shift_fast_prepare __C(R,_ll_dbpol_shift_fast_prepare)
#define ll_dbpol_shift_fast_rec __C(R,_ll_dbpol_shift_fast_rec)
#define ll_dbpol_shift_in_X __C(R,_ll_dbpol_shift_in_X)
#define ll_dbpol_shift_naive __C(R,_ll_dbpol_shift_naive)
#define ll_die __C(R,_ll_die)
#define ll_free __C(R,_ll_free)
#define ll_koetter __C(R,_ll_koetter)
#define ll_lili_vec_list_free __C(R,_ll_lili_vec_list_free)
#define ll_lili_vec_node_attach_after __C(R,_ll_lili_vec_node_attach_after)
#define ll_lili_vec_node_attach_before __C(R,_ll_lili_vec_node_attach_before)
#define ll_lili_vec_node_detach __C(R,_ll_lili_vec_node_detach)
#define ll_lili_vec_node_free __C(R,_ll_lili_vec_node_free)
#define ll_lili_vec_node_malloc __C(R,_ll_lili_vec_node_malloc)
#define ll_lili_vec_node_malloc_copy __C(R,_ll_lili_vec_node_malloc_copy)
#define ll_log2 __C(R,_ll_log2)
#define ll_malloc __C(R,_ll_malloc)
#define ll_mat_copy __C(R,_ll_mat_copy)
#define ll_mat_free __C(R,_ll_mat_free)
#define ll_mat_malloc __C(R,_ll_mat_malloc)
#define ll_mat_malloc_random __C(R,_ll_mat_malloc_random)
#define ll_mat_malloc_zero __C(R,_ll_mat_malloc_zero)
#define ll_mat_transpose __C(R,_ll_mat_transpose)
#define ll_max __C(R,_ll_max)
#define ll_min __C(R,_ll_min)
#define ll_prev_log2 __C(R,_ll_prev_log2)
#define ll_print __C(R,_ll_print)
#define ll_realloc __C(R,_ll_realloc)
#define ll_upol_gcd __C(R,_ll_upol_gcd)
#define ll_upol_gcd_naive __C(R,_ll_upol_gcd_naive)
#define ll_upol_horner __C(R,_ll_upol_horner)
#define ll_upol_malloc_random __C(R,_ll_upol_malloc_random)
#define ll_upol_mul __C(R,_ll_upol_mul)
#define ll_upol_mul_by_X __C(R,_ll_upol_mul_by_X)
#define ll_upol_mul_by_Xn __C(R,_ll_upol_mul_by_Xn)
#define ll_upol_mul_by_affine __C(R,_ll_upol_mul_by_affine)
#define ll_upol_mul_naive __C(R,_ll_upol_mul_naive)
#define ll_upol_multi_horner __C(R,_ll_upol_multi_horner)
#define ll_upol_normalize __C(R,_ll_upol_normalize)
#define ll_upol_print __C(R,_ll_upol_print)
#define ll_upol_quo __C(R,_ll_upol_quo)
#define ll_upol_quo_naive __C(R,_ll_upol_quo_naive)
#define ll_upol_quorem __C(R,_ll_upol_quorem)
#define ll_upol_quorem_naive __C(R,_ll_upol_quorem_naive)
#define ll_upol_random __C(R,_ll_upol_random)
#define ll_upol_rem __C(R,_ll_upol_rem)
#define ll_upol_rem_naive __C(R,_ll_upol_rem_naive)
#define ll_upol_roots __C(R,_ll_upol_roots)
#define ll_upol_shift __C(R,_ll_upol_shift)
#define ll_upol_shift_car2 __C(R,_ll_upol_shift_car2)
#define ll_upol_shift_car22 __C(R,_ll_upol_shift_car22)
#define ll_upol_shift_car2_finish __C(R,_ll_upol_shift_car2_finish)
#define ll_upol_shift_car2_prepare __C(R,_ll_upol_shift_car2_prepare)
#define ll_upol_shift_car2_rec __C(R,_ll_upol_shift_car2_rec)
#define ll_upol_shift_fast __C(R,_ll_upol_shift_fast)
#define ll_upol_shift_fast2 __C(R,_ll_upol_shift_fast2)
#define ll_upol_shift_fast_finish __C(R,_ll_upol_shift_fast_finish)
#define ll_upol_shift_fast_prepare __C(R,_ll_upol_shift_fast_prepare)
#define ll_upol_shift_fast_rec __C(R,_ll_upol_shift_fast_rec)
#define ll_upol_shift_naive __C(R,_ll_upol_shift_naive)
#define ll_upol_square __C(R,_ll_upol_square)
#define ll_upol_square_naive __C(R,_ll_upol_square_naive)
#define ll_vec_add __C(R,_ll_vec_add)
#define ll_vec_add_mul __C(R,_ll_vec_add_mul)
#define ll_vec_add_mul_by_cte __C(R,_ll_vec_add_mul_by_cte)
#define ll_vec_clear __C(R,_ll_vec_clear)
#define ll_vec_cmp __C(R,_ll_vec_cmp)
#define ll_vec_copy __C(R,_ll_vec_copy)
#define ll_vec_free __C(R,_ll_vec_free)
#define ll_vec_hamming_weight __C(R,_ll_vec_hamming_weight)
#define ll_vec_init __C(R,_ll_vec_init)
#define ll_vec_init_zero __C(R,_ll_vec_init_zero)
#define ll_vec_init_random __C(R,_ll_vec_init_random)
#define ll_vec_iszero __C(R,_ll_vec_iszero)
#define ll_vec_malloc __C(R,_ll_vec_malloc)
#define ll_vec_malloc_copy __C(R,_ll_vec_malloc_copy)
#define ll_vec_malloc_random __C(R,_ll_vec_malloc_random)
#define ll_vec_malloc_zero __C(R,_ll_vec_malloc_zero)
#define ll_vec_mul_by_cte __C(R,_ll_vec_mul_by_cte)
#define ll_vec_print __C(R,_ll_vec_print)
#define ll_vec_random __C(R,_ll_vec_random)
#define ll_vec_random_hamming_weight __C(R,_ll_vec_random_hamming_weight)
#define ll_vec_sub __C(R,_ll_vec_sub)
#define ll_vec_sub_mul __C(R,_ll_vec_sub_mul)
#define ll_vec_sub_sub __C(R,_ll_vec_sub_sub)
#define ll_vec_zero __C(R,_ll_vec_zero)

#endif

