/*
 * This file is part of decoding.
 * Copyright (C) 2012  Guillaume Quintin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <decoding/ring_reset.h>

#define R GF5
#define GF_MAGMA_DECL "k := GF(5)"

typedef int R[1];

void R_clear(R a) {
  (void)a;
}

void R_clears(R a,...) {
  (void)a;
}

void R_init(R a) {
  (void)a;
}

void R_inits(R a,...) {
  (void)a;
}

void R_copy(R r,R a) {
  r[0] = a[0];
}

void R_init_copy(R r,R a) {
  r[0] = a[0];
}

void R_print(FILE *out,R a) {
  fputc('0' + a[0],out);
}

int R_cmp(R a,R b) {
  return (a[0] != b[0]);
}

int R_iszero(R a) {
  return (a[0] == 0);
}

void R_zero(R r) {
  r[0] = 0;
}

void R_random(R r) {
  r[0] = rand() % 5;
}

void R_add(R r,R a,R b) {
  r[0] = (a[0] + b[0]) % 5;
}

void R_sub(R r,R a,R b) {
  r[0] = (a[0] + 5 - b[0]) % 5;
}

void R_sub_sub(R r,R a,R b) {
  r[0] = (r[0] + 10 - a[0] - b[0]) % 5;
}

void R_add_mul(R r,R a,R b) {
  r[0] = (r[0] + (a[0] * b[0])) % 5;
}

void R_sub_mul(R r,R a,R b) {
  r[0] = (r[0] + 25 - (a[0] * b[0])) % 5;
}

void R_neg(R r,R a) {
  r[0] = (5 - a[0]) % 5;
}

void R_mul(R r,R a,R b) {
  r[0] = (a[0] * b[0]) % 5;
}

void R_square(R r,R a) {
  r[0] = (a[0] * a[0]) % 5;
}

int R_inv(R r,R a) {
  switch(a[0]) {
    case 1:
      r[0] = 1;
      return 0;
    case 2:
      r[0] = 3;
      return 0;
    case 3:
      r[0] = 2;
      return 0;
    case 4:
      r[0] = 4;
      return 0;
    default:
      return -1;
  }
}

int R_div(R r,R a,R b) {
  R c;
  if ( R_inv(c,b) < 0 ) return -1;
  R_mul(r,a,c);
  return 0;
}

void R_castint(R r,int a) {
  if ( a >= 0 ) r[0] = a % 5; else r[0] = 5 - ((-a) % 5);
}

#include <decoding/algos.h>

