/*
 * Copyright (C) 2012 Guillaume Quintin.
 * This file is part of funcdecl.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define _POSIX_C_SOURCE 1

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define VERSION "funcdecl version 0.1"

void finish_char(FILE *in) {
  int c;
  for(;;) {
    c = fgetc(in);
    if ( c == EOF || c == '\'' ) return;
    if ( c == '\\' ) (void)fgetc(in);
  }
}

void finish_string(FILE *in) {
  int c;
  for(;;) {
    c = fgetc(in);
    if ( c == EOF || c == '\"' ) return;
    if ( c == '\\' ) (void)fgetc(in);
  }
}

void finish_macro(FILE *in) {
  int c;
  for(;;) {
    c = fgetc(in);
    if ( c == EOF || c == '\n' ) return;
    if ( c == '\\' ) (void)fgetc(in);
  }
}

void finish_c99_comment(FILE *in) {
  int c;
  for(;;) {
    c = fgetc(in);
    if ( c == EOF || c == '\n' ) return;
  }
}

void finish_c89_comment(FILE *in) {
  int c;
  for(;;) {
    c = fgetc(in);
    if ( c == EOF ) return;
    if ( c == '*' ) {
      if ( fgetc(in) == '/' ) return;
    }
  }
}

void compile(FILE *in) {
  int c,brace,shit;
  shit = 1;
  brace = 0;
  for(;;) {
    c = fgetc(in);
    switch(c) {
      case EOF:
        return;
      case '#':
        finish_macro(in);
        continue;
      case '\"':
        finish_string(in);
        shit = 0;
        continue;
      case '\'':
        finish_char(in);
        shit = 0;
        continue;
      case '/': {
        int d;
        d = fgetc(in);
        if ( d == '*' ) {
          finish_c89_comment(in);
        }
        else if ( d == '/' ) {
          finish_c99_comment(in);
        }
        else {
          if ( brace == 0 ) {
            fputc(c,stdout);
            fputc(d,stdout);
          }
        }
        continue;
      }
      case '{':
        if ( brace == 0 ) fputc('\n',stdout);
        brace++;
        continue;
      case '}':
        brace--;
        continue;
      case '\n':
      case ';':
      case ' ':
      case '\t':
        if ( brace == 0 && shit == 0 ) fputc(' ',stdout);
        shit = 1;
        continue;
      default:
        if ( brace == 0 ) {
          fputc(c,stdout);
          shit = 0;
        }
        continue;
    }
  }
}

int main(int argc,char **argv) {
  int i;
  FILE *in;
  if ( argc == 2 && !strcmp(argv[1],"-v") ) {
    puts(VERSION);
    return 0;
  }
  if ( argc > 1 ) {
    for( i = 1 ; i < argc ; i++ ) {
      in = fopen(argv[i],"r");
      if ( in ) {
        compile(in);
        fclose(in);
      }
    }
  }
  else {
    compile(stdin);
    fclose(stdin);
  }
  fflush(stdout);
  return 0;
}

