/*
 * This file is part of decoding.
 * Copyright (C) 2012  Guillaume Quintin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#define _POSIX_C_SOURCE 1

#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>

#include <decoding/rings/GF2.c>
#include <decoding/algos.c>

void mtest_dbpol_mul_naive(FILE *out,int n1,int n2) {
  int i;
  R *f,*g,*fg;
  f = ll_dbpol_malloc_random(n2,n2);
  g = ll_dbpol_malloc_random(n2,n2);
  fg = ll_mat_malloc(2 * n2 - 1,2 * n2 - 1);
  for( i = n1 ; i <= n2 ; i++ ) {
    ll_dbpol_mul_naive(fg,f,i,i,g,i,i);
    ll_print(out,"if e eq 0 then\n"
                 "  f  := P ! (%q);\n"
                 "  g  := P ! (%q);\n"
                 "  fg := P ! (%q);\n"
                 "  if not (fg eq (f * g)) then\n"
                 "    print \"Error\";\n"
                 "    e := 1;\n"
                 "  end if;\n"
                 "end if;\n\n",
                 f,i,i,g,i,i,fg,2 * i - 1,2 * i - 1);
  }
  ll_vec_free(f,n2);
  ll_vec_free(g,n2);
  ll_mat_free(fg,2 * n2 - 1,2 * n2 - 1);
}

void mtest_dbpol_mul_by_affine(FILE *out,int n1,int n2) {
  int i;
  R *f,*g,b;
  R_init(b);
  f = ll_dbpol_malloc_random(n2,n2);
  g = ll_mat_malloc(n2,n2 + 1);
  for( i = n1 ; i <= n2 ; i++ ) {
    R_random(b);
    ll_dbpol_mul_by_affine(g,f,i,i,b);
    ll_print(out,"if e eq 0 then\n"
                 "  f := P ! (%q);\n"
                 "  g := P ! (%q);\n"
                 "  b := k ! (%e);\n"
                 "  if not (f * (Y + b) eq g) then\n"
                 "    print \"Error\";\n"
                 "    e := 1;\n"
                 "  end if;\n"
                 "end if;\n\n",
                 f,i,i,g,i,i + 1,&b);
  }
  ll_mat_free(f,n2,n2);
  ll_mat_free(g,n2,n2 + 1);
  R_clear(b);
}

void mtest_dbpol_mul_by_affine_inplace(FILE *out,int n1,int n2) {
  int i;
  R *f,*g,b;
  R_init(b);
  f = ll_dbpol_malloc_random(n2,n2);
  g = ll_mat_malloc(n2,n2 + 1);
  for( i = n1 ; i <= n2 ; i++ ) {
    R_random(b);
    ll_mat_copy(g,f,i,i);
    ll_dbpol_mul_by_affine(g,g,i,i,b);
    ll_print(out,"if e eq 0 then\n"
                 "  f := P ! (%q);\n"
                 "  g := P ! (%q);\n"
                 "  b := k ! (%e);\n"
                 "  if not (f * (Y + b) eq g) then\n"
                 "    print \"Error\";\n"
                 "    e := 1;\n"
                 "  end if;\n"
                 "end if;\n\n",
                 f,i,i,g,i,i + 1,&b);
  }
  ll_mat_free(f,n2,n2);
  ll_mat_free(g,n2,n2 + 1);
  R_clear(b);
}

void mtest_dbpol_shift_naive(FILE *out,int n1,int n2) {
  int i;
  R *f,*g,x,y;
  f = ll_dbpol_malloc_random(n2,n2);
  g = ll_mat_malloc(n2,n2);
  R_init(x);
  R_init(y);
  for( i = n1 ; i <= n2 ; i++ ) {
    R_random(x);
    R_random(y);
    ll_dbpol_shift_naive(g,f,i,i,x,y);
    ll_print(out,"if e eq 0 then\n"
                 "  f := P ! (%q);\n"
                 "  x := k ! (%e);\n"
                 "  y := k ! (%e);\n"
                 "  g := P ! (%q);\n"
                 "  if not (Evaluate(f,[X + x,Y + y]) eq g) then\n"
                 "    print \"Error\";\n"
                 "    e := 1;\n"
                 "  end if;\n"
                 "end if;\n\n",
                 f,i,i,&x,&y,g,i,i);
  }
  ll_mat_free(f,n2,n2);
  ll_mat_free(g,n2,n2);
  R_clear(y);
  R_clear(x);
}

void mtest_dbpol_shift_fast(FILE *out,int n1,int n2) {
  int i;
  R *f,*g,x,y;
  f = ll_dbpol_malloc_random(n2,n2);
  g = ll_mat_malloc(n2,n2);
  R_init(x);
  R_init(y);
  for( i = n1 ; i <= n2 ; i++ ) {
    R_random(x);
    R_random(y);
    ll_dbpol_shift_fast(g,f,i,i,x,y);
    ll_print(out,"if e eq 0 then\n"
                 "  f := P ! (%q);\n"
                 "  x := k ! (%e);\n"
                 "  y := k ! (%e);\n"
                 "  g := P ! (%q);\n"
                 "  if not (Evaluate(f,[X + x,Y + y]) eq g) then\n"
                 "    print \"Error\";\n"
                 "    e := 1;\n"
                 "  end if;\n"
                 "end if;\n\n",
                 f,i,i,&x,&y,g,i,i);
  }
  ll_mat_free(f,n2,n2);
  ll_mat_free(g,n2,n2);
  R_clear(y);
  R_clear(x);
}

void mtest_dbpol_shift_car2(FILE *out,int n1,int n2) {
  int i;
  R *f,*g,x,y;
  f = ll_dbpol_malloc_random(n2,n2);
  g = ll_mat_malloc(n2,n2);
  R_init(x);
  R_init(y);
  for( i = n1 ; i <= n2 ; i++ ) {
    R_random(x);
    R_random(y);
    ll_dbpol_shift_car2(g,f,i,i,x,y);
    ll_print(out,"if e eq 0 then\n"
                 "  f := P ! (%q);\n"
                 "  x := k ! (%e);\n"
                 "  y := k ! (%e);\n"
                 "  g := P ! (%q);\n"
                 "  if not (Evaluate(f,[X + x,Y + y]) eq g) then\n"
                 "    print \"Error\";\n"
                 "    e := 1;\n"
                 "  end if;\n"
                 "end if;\n\n",
                 f,i,i,&x,&y,g,i,i);
  }
  ll_mat_free(f,n2,n2);
  ll_mat_free(g,n2,n2);
  R_clear(y);
  R_clear(x);
}

void mtest_dbpol_shift_car2_inplace(FILE *out,int n1,int n2) {
  int i;
  R *f,*g,x,y;
  f = ll_dbpol_malloc_random(n2,n2);
  g = ll_mat_malloc(n2,n2);
  R_init(x);
  R_init(y);
  for( i = n1 ; i <= n2 ; i++ ) {
    R_random(x);
    R_random(y);
    ll_mat_copy(g,f,i,i);
    ll_dbpol_shift_car2(g,g,i,i,x,y);
    ll_print(out,"if e eq 0 then\n"
                 "  f := P ! (%q);\n"
                 "  x := k ! (%e);\n"
                 "  y := k ! (%e);\n"
                 "  g := P ! (%q);\n"
                 "  if not (Evaluate(f,[X + x,Y + y]) eq g) then\n"
                 "    print \"Error\";\n"
                 "    e := 1;\n"
                 "  end if;\n"
                 "end if;\n\n",
                 f,i,i,&x,&y,g,i,i);
  }
  ll_mat_free(f,n2,n2);
  ll_mat_free(g,n2,n2);
  R_clear(y);
  R_clear(x);
}

#define D1 5
#define D2 100

int main(void) {
  srand(time(0));
  ll_print(stdout,"clear;\n"
                   GF_MAGMA_DECL ";\n"
                  "P<X,Y> := PolynomialRing(k,2);\n"
                  "e := 0;\n\n");
  mtest_dbpol_mul_naive(stdout,D1,D2);
  mtest_dbpol_mul_by_affine(stdout,D1,D2);
  mtest_dbpol_mul_by_affine_inplace(stdout,D1,D2);
  mtest_dbpol_shift_naive(stdout,D1,D2);
  mtest_dbpol_shift_fast(stdout,D1,D2);
  mtest_dbpol_shift_car2(stdout,D1,D2);
  mtest_dbpol_shift_car2_inplace(stdout,D1,D2);
  ll_print(stdout,"if e eq 0 then\n"
                  "  print \"ALL OK\";\n"
                  "end if;\n\n");
  return 0;
}

